------------------------------------------------------
-- SLINK Optical interface sender
--
--  Ver 1.00
--
-- Dominique Gigi Feb 2012
------------------------------------------------------
--   
--  
-- 
--  
------------------------------------------------------
LIBRARY ieee;
--LIBRARY altera_mf;
--LIBRARY altera;


USE ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
--LIBRARY lpm;
--USE lpm.lpm_components.all; 
--USE altera_mf.altera_mf_components.all;
--USE altera.altera_primitives_components.all;

entity SLINK_opt is
port (
	reset		: in std_logic;
	-- FED interface
	SYS_CLK		: in std_logic;
	LINKWe		: in std_logic;
	LINKCtrl	: in std_logic;
	LINKData	: in std_logic_vector(63 downto 0);
	src_ID		: in std_logic_vector(15 downto 0);
	inject_err	: in std_logic_vector(17 downto 0);
	read_CE		: in std_logic;
	Addr		: in std_logic_vector(15 downto 0);
	status_data : out std_logic_vector(63 downto 0);
	LINKDown	: out std_logic;
	LINK_LFF	: out std_logic;
	
	-- interface SERDES
	clock		: in std_logic;
	serdes_init	: in std_logic;
	SD_Data_o	: out std_logic_vector(31 downto 0);
	SD_Kb_o		: out std_logic_vector(3 downto 0);
	clock_r		: in std_logic;	
	SD_Data_i	: in std_logic_vector(31 downto 0);
	SD_Kb_i		: in std_logic_vector(3 downto 0);
	
	ack_cnt		: out std_logic;		-- on transmitter clock
	pckt_cnt	: out std_logic;		-- on transmitter clock
	retransmit	: out std_logic;		-- on transmitter clock
	cnt_evt		: out std_logic			-- pulse for each event (on sys_clk)



	);
end SLINK_opt;

architecture behavioral of SLINK_opt is

component fed_itf
generic (generator : boolean := true);
port (
	reset		: in std_logic;
	sys_clk		: in std_logic;
	LinkWe 		: in  STD_LOGIC;
	LinkCtrl 	: in  STD_LOGIC;
	LinkData 	: in  STD_LOGIC_VECTOR (63 downto 0);
	LinkAlmostFull : out  STD_LOGIC;
	LinkDown 	: out  STD_LOGIC;
	src_ID 		: in  STD_LOGIC_VECTOR (15 downto 0);
	inject_err 	: in  STD_LOGIC_VECTOR (17 downto 0);
	read_ce 	: in  STD_LOGIC;
	addr 		: in  STD_LOGIC_VECTOR (15 downto 0);
	status_data : out  STD_LOGIC_VECTOR (63 downto 0);
	clock		: in std_logic; -- clock from internal logic
	block_free	: in std_logic;	-- almost one block is free
	data_fed	: out std_logic_vector(63 downto 0);
	block_sz_fed: out std_logic_vector(15 downto 0);
	wr_ena		: out std_logic;
	start_evt	: out std_logic;  
	stop_evt	: out std_logic;  
	end_blk_fed	: out std_logic;
	wr_cmd		: in std_logic;  
	func		: in std_logic_vector(15 downto 0);
	data_wr		: in std_logic_vector(31 downto 0);
	data_rd		: out std_logic_vector(31 downto 0);
	cnt_evt		: out std_logic			-- pulse for each event (on sys_clk)
	);
end component;

component Core_logic  
port (
	reset		: in std_logic;
	clock		: in std_logic;
	low_clk		: in std_logic;
		-- interface from the FED block	
	data_fed	: in std_logic_vector(63 downto 0);
	wen			: in std_logic;
	start_evt	: in std_logic;
	stop_evt	: in std_logic;
	block_sz_fed: in std_logic_vector(15 downto 0);
	end_blk_fed	: in std_logic;
	block_free	: out std_logic;
	src_ID		: in std_logic_vector(15 downto 0);
	--req_resync_slink	: in std_logic;
		-- interface to the SERDES OUT (send part)
	start_pckt	: out std_logic; -- trigger the packet send
	init_pckt	: out std_logic; -- indicates that the packet is a Init packet
	ack_pckt	: out std_logic; -- indicates that the packet is a acknoldge packet
	data_pckt	: out std_logic; -- indicates that the packet is a data packet
	data_evt	: out std_logic_vector(63 downto 0);	--data for data packet
	status		: out std_logic_vector(63 downto 0); --status data for acknowledge packet
	card_ID		: out std_logic_vector(15 downto 0); -- CARD_ID
	Seq_nb		: out std_logic_vector(30 downto 0); -- sequence number
	len_pckt	: out std_logic_vector(15 downto 0); -- length of the packet (for data packet only) other 0
	cmd			: out std_logic_vector(63 downto 0); -- command bit for data packet only
	rd_dt		: in std_logic;
	end_snd_pckt	: in std_logic;
	idle_state	: in std_logic;
	serdes_init	: in std_logic;
		-- interface to the SERDES IN (receiver part)
	cmd_rcv		: in std_logic_vector(31 downto 0);	-- command from MOL
	data_rcv	: in std_logic_vector(31 downto 0);	-- data from MOL
	ena_cmd		: in std_logic; -- validate command
	sta_dt		: in std_logic_vector(63 downto 0);	-- value return in an ack packet
	ena_ack		: in std_logic;		
	seqnb_rcv 		: in std_logic_vector(30 downto 0);	-- seq numb from cmd (need an ack)
	card_ID_rcv	: in std_logic_vector(15 downto 0); 
	retransmit	: out std_logic;
	wr_cmd		: out std_logic;  
	func		: out std_logic_vector(15 downto 0);
	data_wr		: out std_logic_vector(31 downto 0);
	data_rd		: in std_logic_vector(31 downto 0)
	);
end component;

component build_pckt_s  
port (
	reset		: in std_logic;
	clock		: in std_logic;
	
	start_pckt	: in std_logic;	-- trigger the packet send
	init_pckt	: in std_logic; -- indicates that the packet is a Init packet
	ack_pckt	: in std_logic; -- indicates that the packet is a acknoldge packet
	data_pckt	: in std_logic; -- indicates that the packet is a data packet
	data_evt	: in std_logic_vector(63 downto 0);	--data for data packet
	status		: in std_logic_vector(63 downto 0); --status data for acknowledge packet
	card_ID		: in std_logic_vector(15 downto 0); -- CARD_ID
	Seq_nb		: in std_logic_vector(30 downto 0); -- sequence number
	len_pckt	: in std_logic_vector(15 downto 0); -- length of the packet (for data packet only) other 0
	cmd			: in std_logic_vector(63 downto 0); -- command bit for data packet only
	error_gen	: in std_logic_vector(3 downto 0);
	rd_dt		: out std_logic;					-- request data for data packet only
	end_pckt	: out std_logic;
		
	datao		: out std_logic_vector(31 downto 0); --- data and K bit send to SERDES
	k_byte		: out std_logic_vector( 3 downto 0);
	idle_state	: out std_logic 	);
end component;

component rcv_pckt_s 
port (
	reset		: in std_logic;
	clock		: in std_logic;
	clock_t		: in std_logic;
	
	datai		: in std_logic_vector(31 downto 0); --- data and K bit send from SERDES
	k_byte		: in std_logic_vector( 3 downto 0);
	
	error_gen	: in std_logic;
	
	cmd			: out std_logic_vector(31 downto 0);	-- command from MOL
	data		: out std_logic_vector(31 downto 0);	-- data from MOL
	ena_cmd		: out std_logic; -- validate command
	
	status		: out std_logic_vector(63 downto 0);	-- value return from read command request
	seqnb 		: out std_logic_vector(30 downto 0);	-- seq numb from cmd (need an ack)
	ena_ack		: out std_logic;		
	
	card_ID		: out std_logic_vector(15 downto 0) 	);
end component;

SIGNAL	data_rcv	:std_logic_vector(31 downto 0);
SIGNAL	ena_cmd		:std_logic;
SIGNAL	sta_dt		:std_logic_vector(63 downto 0);
SIGNAL	ena_ack		:std_logic;
SIGNAL	seqnb_rcv	:std_logic_vector(30 downto 0);
SIGNAL	idle_state	:std_logic;
SIGNAL	end_snd_pckt:std_logic;
SIGNAL	rd_dt		:std_logic;
SIGNAL	cmd_rcv		:std_logic_vector(31 downto 0);
SIGNAL	cmd			:std_logic_vector(63 downto 0);
SIGNAL	len_pckt	:std_logic_vector(15 downto 0);
SIGNAL	Seq_nb		:std_logic_vector(30 downto 0);
SIGNAL	card_ID_rcv	:std_logic_vector(15 downto 0);
SIGNAL	card_ID_snd	:std_logic_vector(15 downto 0);
SIGNAL	status		:std_logic_vector(63 downto 0);
SIGNAL	data_evt	:std_logic_vector(63 downto 0);
SIGNAL	data_pckt	:std_logic;
SIGNAL	ack_pckt	:std_logic;
SIGNAL	init_pckt	:std_logic;
SIGNAL	start_pckt	:std_logic;
SIGNAL	end_blk_fed	:std_logic;
SIGNAL	stop_evt	:std_logic;
SIGNAL	start_evt	:std_logic;
SIGNAL	wr_ena		:std_logic;
SIGNAL	block_sz_fed:std_logic_vector(15 downto 0);
SIGNAL  data_fed	:std_logic_vector(63 downto 0);
SIGNAL  block_free	:std_logic;
		-- interface slave to read and write
SIGNAL  wr_cmd		: std_logic;  
SIGNAL  func		: std_logic_vector(15 downto 0);
SIGNAL  data_wr		: std_logic_vector(31 downto 0);
SIGNAL  data_rd		: std_logic_vector(31 downto 0);

--******************************************************************
--*******************  BEGIN  **************************************
--******************************************************************
begin

i1:fed_itf
port map(
	reset			=> reset,
	sys_clk			=> SYS_CLK,
	LinkWe 			=> LINKWe,
	LinkCtrl 		=> LinkCtrl,
	LinkData 		=> LinkData,
	LinkAlmostFull	=> LINK_LFF,
	LinkDown 		=> LINKDown,
	src_ID 			=> src_ID,
	inject_err 		=> inject_err,
	read_ce 		=> read_CE,
	addr 			=> Addr,
	status_data 	=> status_data,
	clock			=> clock,
	block_free		=> block_free,
	data_fed		=> data_fed,
	block_sz_fed	=> block_sz_fed,
	wr_ena			=> wr_ena,
	start_evt		=> start_evt,
	stop_evt		=> stop_evt,
	end_blk_fed		=> end_blk_fed,
	wr_cmd			=> wr_cmd,  
	func			=> func,
	data_wr			=> data_wr,
	data_rd			=> data_rd,
	cnt_evt			=> cnt_evt);

i2:Core_logic  
port map(
	reset			=> reset,
	low_clk			=> clock,
	clock			=> clock,
		-- interface from the FED block	
	data_fed		=> data_fed,
	wen				=> wr_ena,
	start_evt		=> start_evt,
	stop_evt		=> stop_evt,
	block_sz_fed	=> block_sz_fed,
	end_blk_fed		=> end_blk_fed,
	block_free		=> block_free,
	src_ID			=> src_ID,
	--req_resync_slink	=> Addr(0),
		-- interface to the SERDES OUT (send part)
	start_pckt		=> start_pckt,
	init_pckt		=> init_pckt,
	ack_pckt		=> ack_pckt,
	data_pckt		=> data_pckt,
	data_evt		=> data_evt,
	status			=> status,
	card_ID			=> card_ID_snd,
	Seq_nb			=> Seq_nb,
	len_pckt		=> len_pckt,
	cmd				=> cmd,
	rd_dt			=> rd_dt,
	end_snd_pckt	=> end_snd_pckt,
	idle_state		=> idle_state,
	serdes_init		=> serdes_init,
		-- interface to the SERDES IN (receiver part)
	cmd_rcv			=> cmd_rcv,
	data_rcv		=> data_rcv,
	ena_cmd			=> ena_cmd,
	sta_dt			=> sta_dt,
	ena_ack			=> ena_ack,
	seqnb_rcv 		=> seqnb_rcv,
	card_ID_rcv		=> card_ID_rcv,
	retransmit		=> retransmit,
	wr_cmd			=> wr_cmd,  
	func			=> func,
	data_wr			=> data_wr,
	data_rd			=> data_rd); 

i3:build_pckt_s  
port map(
	reset		=> reset,
	clock		=> clock,
	
	start_pckt	=> start_pckt,
	init_pckt	=> init_pckt,
	ack_pckt	=> ack_pckt,
	data_pckt	=> data_pckt,
	data_evt	=> data_evt,
	status		=> status,
	card_ID		=> card_ID_snd,
	Seq_nb		=> Seq_nb,
	len_pckt	=> len_pckt,
	cmd			=> cmd,
	error_gen	=> Addr(4 downto 1),
	rd_dt		=> rd_dt,
	end_pckt	=> end_snd_pckt,
		
	datao		=> SD_Data_o,
	k_byte		=> SD_Kb_o,
	idle_state	=> idle_state	);

i4:rcv_pckt_s 
port map(
	reset		=> reset,
	clock		=> clock_r,
	clock_t		=> clock,
	datai		=> SD_Data_i,
	k_byte		=> SD_Kb_i,
	error_gen	=> Addr(5),
	cmd			=> cmd_rcv,
	data		=> data_rcv,
	ena_cmd		=> ena_cmd,
	status		=> sta_dt,
	ena_ack		=> ena_ack,
	seqnb 		=> seqnb_rcv,
	card_ID		=> card_ID_rcv 	);
	
	
ack_cnt <= 	ena_ack;
pckt_cnt <= end_snd_pckt;
end behavioral;


